//
//  CMDAwesomeButton.m
//  CMDAwesomeButton
//
//  Created by Caleb Davenport on 4/9/14.
//  Copyright (c) 2014 Caleb Davenport. All rights reserved.
//

#import "CMDAwesomeButton.h"

@interface CMDAwesomeButton ()

@property (nonatomic, readonly) NSMutableDictionary *storage;

@end

@implementation CMDAwesomeButton

#pragma mark - Properties

@synthesize storage = _cmd_storage;
@synthesize backgroundView = _cmd_backgroundView;

- (NSMutableDictionary *)storage {
    if (!_cmd_storage) {
        _cmd_storage = [NSMutableDictionary new];
    }
    return _cmd_storage;
}

- (void)setBackgroundView:(UIView *)view {
    [_cmd_backgroundView removeFromSuperview];
    _cmd_backgroundView = view;
    _cmd_backgroundView.userInteractionEnabled = NO;
    [self insertSubview:_cmd_backgroundView atIndex:0];
    [self setNeedsLayout];
}


#pragma mark - UIView

- (void)layoutSubviews {
    [super layoutSubviews];
    self.backgroundView.frame = self.bounds;
}


#pragma mark - UIControl

- (void)setHighlighted:(BOOL)highlighted {
    [super setHighlighted:highlighted];
    [self applyStorageForCurrentState];
}

- (void)setSelected:(BOOL)selected {
    [super setSelected:selected];
    [self applyStorageForCurrentState];
}

- (void)setEnabled:(BOOL)enabled {
    [super setEnabled:enabled];
    [self applyStorageForCurrentState];
}


#pragma mark - Public

- (void)setValue:(id)value forKeyPath:(NSString *)keyPath state:(UIControlState)state {
    NSMutableDictionary *storage = [self storageForControlState:state];
    storage[keyPath] = value;
    [self applyStorageForCurrentState];
}

- (void)setBackgroundColor:(UIColor *)color forState:(UIControlState)state {
    [self setValue:color forKeyPath:@"backgroundColor" state:state];
}

- (void)setBorderColor:(UIColor *)color forState:(UIControlState)state {
    [self setValue:(id)[color CGColor] forKeyPath:@"layer.borderColor" state:state];
}

- (void)setTintColor:(UIColor *)color forState:(UIControlState)state {
    [self setValue:color forKeyPath:@"tintColor" state:state];
}

- (void)setBorderWidth:(CGFloat)width forState:(UIControlState)state {
    [self setValue:@(width) forKeyPath:@"layer.borderWidth" state:state];
}

- (void)setCornerRadius:(CGFloat)radius forState:(UIControlState)state {
    [self setValue:@(radius) forKeyPath:@"layer.cornerRadius" state:state];
}


#pragma mark - Private

- (NSMutableDictionary *)storageForControlState:(UIControlState)state {
    NSString *key = [@(state) stringValue];
    NSMutableDictionary *dictionary = self.storage[key];
    if (!dictionary) {
        dictionary = [NSMutableDictionary new];
        self.storage[key] = dictionary;
    }
    return dictionary;
}

- (void)applyStorageForCurrentState {
    NSDictionary *defaultStateStorage = [self storageForControlState:UIControlStateNormal];
    NSDictionary *currentStateStorage = [self storageForControlState:self.state];
    NSMutableDictionary *mergedStorage = [NSMutableDictionary new];
    [mergedStorage addEntriesFromDictionary:defaultStateStorage];
    [mergedStorage addEntriesFromDictionary:currentStateStorage];
    [mergedStorage enumerateKeysAndObjectsUsingBlock:^(NSString *key, id object, BOOL *stop) {
        [self setValue:object forKeyPath:key];
    }];
}

@end
